/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Matteo Frigo, mfrigo@us.ibm.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "simplex.h"

/**************************************************************/
/* Variables read from the input file: */

/* The symmetry of the filter */
enum symmetry symmetry;

/* The sampling frequency */
double sampling_frequency;

/* The length of the filter */
int filter_length;

/* A linked list of specifications */
SPECIFICATION *specifications;

/**************************************************************/
/* Parser: */

void parse(void)
{
     switch (getchar()) {
	 case 'E':
	 case 'e':
	      symmetry = EVEN;
	      break;
	 case 'O':
	 case 'o':
	      symmetry = ODD;
	      break;
	 default:
	      goto parse_error;
     }

     if (scanf("%d", &filter_length) != 1)
	      goto parse_error;
     
     if (scanf("%lg", &sampling_frequency) != 1)
	      goto parse_error;

     specifications = NULL;
     for (;;) {
	  double left_freq, right_freq, left_val, right_val, weight;
	  int n;
	  SPECIFICATION *s;

	  n = scanf("%lg%lg%lg%lg%lg",
		    &left_freq, &right_freq,
		    &left_val, &right_val,
		    &weight);

	  if (n == EOF)
	       break;
	  if (n != 5)
	       goto parse_error;

	  s = malloc(sizeof(*s));
	  s->left_freq = left_freq / sampling_frequency;
	  if (s->left_freq < 0 || s->left_freq > .5)
	       goto wrong_frequency;
	  if (s->right_freq < 0 || s->right_freq > .5)
	       goto wrong_frequency;
	  s->right_freq = right_freq / sampling_frequency;
	  s->left_val = left_val;
	  s->right_val = right_val;
	  s->weight = weight;
	  
	  /* append to list of specifications */
	  s->next = specifications;
	  specifications = s;
     }

     return;

 parse_error:
     fprintf(stderr, "parse error\n");
     exit(EXIT_FAILURE);

 wrong_frequency:
     fprintf(stderr, "wrong frequency\n");
     exit(EXIT_FAILURE);

}

/**************************************************************/

int main(int argc, char *argv[])
{
     parse();
     solve();

     return EXIT_SUCCESS;
}
