/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Matteo Frigo, mfrigo@us.ibm.com
*/

#ifndef SIMPLEX_H
#define SIMPLEX_H

#ifdef __cplusplus
extern "C" {
#endif

enum simplex_result {
     S_OPTIMAL, 
     S_INFEASIBLE,
     S_UNBOUNDED, 
     S_ITERATION_LIMIT
};

enum symmetry { 
  EVEN,
  ODD 
};

typedef struct specification {
     double left_freq, right_freq;
     double left_val, right_val;
     double weight;
     struct specification *next;
} SPECIFICATION;

typedef struct {
     /* public fields */
     enum simplex_result result;
     double *primals;                  /* primal variables */
     double *duals;                    /* dual variables */
     double objective_value;
     int iteration_limit;
     double *obj;                      /* objective function */

     /* private fields */
     int maxnrow;
     int ncol;
     double *A;
     double *rhs;
     int nrow;
} SIMPLEX;

SIMPLEX *simplex_make(int nrow, int ncol);
void simplex_free(SIMPLEX *self);
void simplex_add_equation(SIMPLEX *self, double *row, double rhs);
void simplex_set_objective(SIMPLEX *self, double *row);
enum simplex_result simplex_solve(SIMPLEX *self);
void simplex_set_iteration_limit(SIMPLEX *self, int l);
void solve(void);

/* the symmetry of the filter */
extern enum symmetry symmetry;

/* the sampling frequency */
extern double sampling_frequency;

/* the length of the filter */
extern int filter_length;

/* a list of specifications */
extern SPECIFICATION *specifications;

#ifdef __cplusplus
}
#endif


#endif
