/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Geert Janssen, geert@us.ibm.com
*/

#include "bdd_api.h"

/* -------------------------------------------------------------------------*/
/*			    BDD Package Interface			    */
/* -------------------------------------------------------------------------*/

BDD_Mgr
Bdd_init(void)
{
  BDD_Mgr mgr;

  mgr = Cudd_Init(0, 0, 512, CUDD_CACHE_SLOTS, 32<<20);
  Cudd_AutodynEnable(mgr, CUDD_REORDER_SIFT);
  return mgr;
}

void
Bdd_quit(BDD_Mgr m)
{
  Cudd_Quit(m);
}

BDD
Bdd_dup(BDD_Mgr m, BDD f)
{
  Cudd_Ref(f);
  return f;
}

int
Bdd_0_p(BDD_Mgr m, BDD f)
{
  return f == Cudd_ReadLogicZero(m);
}

BDD
Bdd_0(BDD_Mgr m)
{
  return Bdd_dup(m, Cudd_ReadLogicZero(m));
}

int
Bdd_1_p(BDD_Mgr m, BDD f)
{
  return f == Cudd_Not(Cudd_ReadLogicZero(m));
}

BDD
Bdd_1(BDD_Mgr m)
{
  return Bdd_dup(m, Cudd_ReadOne(m));
}

BDD
Bdd_var(BDD_Mgr m, int v)
{
  return Bdd_dup(m, Cudd_bddIthVar(m, v));
}

BDD
Bdd_not(BDD_Mgr m, BDD f)
{
  return Bdd_dup(m, Cudd_Not(f));
}

BDD
Bdd_and(BDD_Mgr m, BDD f, BDD g)
{
  return Bdd_dup(m, Cudd_bddAnd(m, f, g));
}

BDD
Bdd_and_vec(BDD_Mgr m, BDD A[], int n)
{
  BDD R = Bdd_1(m);
  int i;

  for (i = 0; i < n; i++) {
    BDD s;

    R = Bdd_and(m, A[i], s=R);
    Bdd_free(m, s);
  }
  return R;
}

BDD
Bdd_or(BDD_Mgr m, BDD f, BDD g)
{
  return Bdd_dup(m, Cudd_bddOr(m, f, g));
}

BDD
Bdd_or_vec(BDD_Mgr m, BDD A[], int n)
{
  BDD R = Bdd_0(m);
  int i;

  for (i = 0; i < n; i++) {
    BDD s;

    R = Bdd_or(m, A[i], s=R);
    Bdd_free(m, s);
  }
  return R;
}

BDD
Bdd_equiv(BDD_Mgr m, BDD f, BDD g)
{
  return Bdd_dup(m, Cudd_bddXnor(m, f, g));
}

void
Bdd_free(BDD_Mgr m, BDD f)
{
  if (f != NULL)
    Cudd_DelayedDerefBdd(m, f);
}

void
Bdd_free_vec(BDD_Mgr m, BDD A[], int n)
{
  int i;

  for (i = 0; i < n; i++)
    Bdd_free(m, A[i]);
}

int
Bdd_nr_minterms(BDD_Mgr m, BDD f, int n)
{
  return (int) Cudd_CountMinterm(m, f, n);
}

BDD
Bdd_swap_vars(BDD_Mgr m, BDD f, BDD *x, BDD *y, int n)
{
  return Bdd_dup(m, Cudd_bddSwapVariables(m, f, x, y, n));
}

BDD
Bdd_exist(BDD_Mgr m, BDD f, BDD vs)
{
  return Bdd_dup(m, Cudd_bddExistAbstract(m, f, vs));
}

BDD
Bdd_univ(BDD_Mgr m, BDD f, BDD vs)
{
  return Bdd_dup(m, Cudd_bddUnivAbstract(m, f, vs));
}
