/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Geert Janssen, geert@us.ibm.com

   You only need to complete 3 functions in this file.
   These functions are clearly marked by the text

   YOU NEED TO IMPLEMENT THIS BODY !!!

   as a comment in their body. But feel free to change anything else
   if you think it suits you better.

   Good luck.
*/

#include <assert.h>
#include <errno.h>
#include <ctype.h>

#include "parser.h"
#include "bdd_api.h"
#include "relation.h"

/*#define DEBUG*/

/* The model collects all data necessary to efficiently analyze the
   state machine.
*/
typedef struct {
  BDD_Mgr mgr;			/* BDD manager */
  int n;			/* number of state variables */
  /* elements are in order of occurrence in the input */
  BDD *CV;			/* current-state variables CV[i] ~ NF[i] */
  BDD *NF;			/* next-state functions */
  char *VN;			/* variable name */
} Model_S, *Model;

/* -------------------------------------------------------------------------*/
/*			    Model Construction				    */
/* -------------------------------------------------------------------------*/

/* First 26 BDD variables are reserved for current-state;
   Starting from BDD variable number 27 we have the primary inputs.
*/

static BDD
get_input_var_bdd(BDD_Mgr mgr, int v)
{
  assert(islower(v));
  return Bdd_var(mgr, (v - 'a') + 27);
}

/* Returns projection function for current-state variable v. */
static BDD
get_currstate_var_bdd(BDD_Mgr mgr, int v)
{
  assert(isupper(v));
  return Bdd_var(mgr, (v - 'A') + 1);
}

/* Returns BDD for expression tree E. */
static BDD
get_expression_bdd(BDD_Mgr mgr, Tree E)
{

  /* YOU NEED TO IMPLEMENT THIS BODY !!! */

  /* Dummy return value for now: */
  return NULL;
}

/* Fill in all fields of model m given input data in tree t.
   mgr must be an initialized BDD manager.
*/
static void
model_build(BDD_Mgr mgr, Tree t, Model m)
{
  Tree tt;
  int i;

#ifdef DEBUG
  printf("Building model...\n");
#endif

  /* Cache BDD manager with model: */
  m->mgr = mgr;

  /* Get n, NF, NV, CV: */

  /* Count number of state variables: */
  for (i = 0, tt = t; tt; i++, tt = T_RIGHT(tt))
    ;
  m->n = i;

#ifdef DEBUG
  printf("The FSM has %d current state variables.\n", i);
#endif

  /* Allocate space for the arrays: */
  m->VN = malloc(m->n * sizeof(char));
  m->CV = malloc(m->n * sizeof(BDD));
  m->NF = malloc(m->n * sizeof(BDD));

  /* Fill in the arrays: */
  for (i = 0; i < m->n; i++, t = T_RIGHT(t)) {
    Tree f = T_LEFT(t);
    int v;

    /* Left of sequence node is a next-state function node: */
    /* Left of next-state function is state variable: */
    v = T_TYPE(T_LEFT(f));

    m->VN[i] = v;
    m->CV[i] = get_currstate_var_bdd(mgr, v);
    m->NF[i] = get_expression_bdd(mgr, T_RIGHT(f));
  }

#ifdef DEBUG
  printf("Building model...done.\n");
#endif
}

static void
model_free(Model m)
{
  free(m->VN);
  Bdd_free_vec(m->mgr, m->CV, m->n);
  Bdd_free_vec(m->mgr, m->NF, m->n);
}

/* --------------------------------------------------------------------------*/
/*		    Maximal Signal Correspondence Relation		     */
/* --------------------------------------------------------------------------*/

/* Returns number of equivalence classes of the equivalence relation R. */
static int
count_classes(int n, char **R)
{

  /* YOU NEED TO IMPLEMENT THIS BODY !!! */

  /* Dummy return value for now: */
  return 0;
}

/* Computes the solution. m is the FSM model. */
static int
compute(Model m)
{

  /* YOU NEED TO IMPLEMENT THIS BODY !!! */

  /* Dummy return value for now: */
  return 0;
}

/* --------------------------------------------------------------------------*/
/*			    Main					     */
/* --------------------------------------------------------------------------*/

int
main(int argc, char *argv[])
{
  Tree T;
  BDD_Mgr mgr;
  Model_S model, *M = &model;

  if (argc > 1 && argv[1])
    if (!freopen(argv[1], "r", stdin)) {
      fprintf(stderr, "Error: Cannot open file %s\n", argv[1]);
      exit(EXIT_FAILURE);
    }

  /* Read the input: */
  T = parse();

  /* Initialize the BDD package: */
  mgr = Bdd_init();

  /* Build the internal FSM model: */
  model_build(mgr, T, M);

  /* Compute and output the solution: */
  printf("%d\n", compute(M));

  /* Free up the model: */
  model_free(M);

  /* Quit the BDD package: */
  Bdd_quit(mgr);

  /* Free up the parse tree: */
  tree_free(T);

  return EXIT_SUCCESS;
}
