/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Geert Janssen, geert@us.ibm.com
*/

#ifndef PARSER_H
#define PARSER_H

#include "tree.h"

#if defined __cplusplus
extern "C" {
#endif

/* Syntax for specifying a finite state transition system.
   The input consists of 1 or more next-state functions, each terminated
   with a semicolon.
   White-space (blank, tab, newline) has no semantic meaning and may be used
   in any quantity to enhance the readability. The same holds for comments
   that start with the hash character '#' and extend to the end of the line.

       Input       : NSFunctions .

       NSFunctions : NSFunction [ NSFunctions ] .

       NSFunction  : "@" S_Variable "=" Expression ";" .

   Syntax of logic expressions.
   Note that the (optional) AND-operator '&' binds stronger than the OR-
   operator '+'. Both are commutative and associative. Note that the
   AND-operator is optional and may thus also be denoted by juxtaposition
   of Factors. The NOT-operator '!' has the highest precedence.

       Expression  : Term [ "+" Expression ] .

       Term        : Factor [ [ "&" ] Term ] .

       Factor      : Primary | "!" Factor .

       Primary     : Constant | Variable | "(" Expression ")" .

       Constant    : "0" | "1" .

       Variable    : S_Variable | I_Variable .

   Note: variables are single letters; case is significant.

       S_Variable  : "A" | "B" | "C" | ... | 'Z' .

       I_Variable  : "a" | "b" | "c" | ... | 'z' .
*/

/* Reads a finite state transition system from stdin and constructs a tree
   using the functions and macros defined in the file `tree.h'.
   The accepted input format is defined above.
   Aborts (with exit(1)) whenever a syntax error occurs.

   The top-level production rule is:

   	Input : NSFunctions .

   The parse tree of NSFunctions is returned by `parse' as a sequence node.
*/
Tree parse(void);

#ifdef __cplusplus
}
#endif

#endif /* PARSER_H */
