/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Geert Janssen, geert@us.ibm.com
*/

#include "relation.h"

/* -------------------------------------------------------------------------*/
/*			    Simple Relation Data Structure		    */
/* -------------------------------------------------------------------------*/

char **
R_alloc(int n)
{
  char **R = malloc(n * sizeof(char *));
  int i;

  for (i = 0; i < n; i++)
    R[i] = malloc(n * sizeof(char));
  return R;
}

void
R_free(int n, char **R)
{
  int i;

  for (i = 0; i < n; i++)
    free(R[i]);
  free(R);
}

void
R_zero(int n, char **R)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < n; j++)
      R[i][j] = 0;
}

void
R_unit(int n, char **R)
{
  int i, j;

  for (i = 0; i < n; i++) {
    R[i][i] = 1;
    for (j = i+1; j < n; j++) {
      R[i][j] = 0;
      R[j][i] = 0;
    }
  }
}

void
R_complete(int n, char **R)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < n; j++)
      R[i][j] = 1;
}

void
R_copy(int n, char **R1, char **R2)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < n; j++)
      R1[i][j] = R2[i][j];
}

int
R_equal(int n, char **R1, char **R2)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < n; j++)
      if (R1[i][j] != R2[i][j])
	return 0;
  return 1;
}

void
R_print(int n, char **R, char names[])
{
  int i, j;

  if (names) {
    printf("  ");
    for (j = 0; j < n; j++)
      printf(" %c", names[j]);
    printf("\n");
  }

  for (i = 0; i < n; i++) {
    if (names)
      printf("%c ", names[i]);
    for (j = 0; j < n; j++)
      if (R[i][j])
	printf(" 1");
      else
	printf(" 0");
    printf("\n");
  }
}
