/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Geert Janssen, geert@us.ibm.com
*/

#ifndef RELATION_H
#define RELATION_H

#include <stdio.h>
#include <stdlib.h>

#if defined __cplusplus
extern "C" {
#endif

/* -------------------------------------------------------------------------*/
/*			    Simple Relation Data Structure		    */
/* -------------------------------------------------------------------------*/

/* A relation is implemented as a bit matrix.
   For an  equivalence relation it would suffice to only store
   the upper-right triangle.
*/

/* Allocate n x n sized relation. */
char **R_alloc(int n);

/* Free up space taken by n x n relation R. */
void R_free(int n, char **R);

/* Initialize n x n relation R to empty relation. */
void R_zero(int n, char **R);

/* Initialize n x n relation R to unit relation. */
void R_unit(int n, char **R);

/* Initialize n x n relation R to complete relation. */
void R_complete(int n, char **R);

/* Copy n x n relation R2 to R1. */
void R_copy(int n, char **R1, char **R2);

/* Tests whether n x n relations R1 and R2 are equal. */
int R_equal(int n, char **R1, char **R2);

/* Prints the relation R as a bit matrix.
   Accept an array of names to use as row/column labels.
   Each name is just a single character.
   When this arg is NULL, no labels are printed.
*/
void R_print(int n, char **R, char names[]);

#if defined __cplusplus
}
#endif

#endif /* RELATION_H */
