/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Geert Janssen, geert@us.ibm.com
*/

/* ------------------------------------------------------------------------ */
/* INCLUDES                                                                 */
/* ------------------------------------------------------------------------ */

#include "tree.h"

/* ------------------------------------------------------------------------ */
/* LOCAL DEFINES                                                            */
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
/* LOCAL TYPE DEFINITIONS                                                   */
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
/* LOCAL VARIABLES                                                          */
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
/* FUNCTION DEFINITIONS                                                     */
/* ------------------------------------------------------------------------ */

void
tree_free(Tree t)
{
  if (t) {
    tree_free(T_LEFT(t));
    tree_free(T_RIGHT(t));
    free(t);
  }
}

Tree
tree_mk(int type, Tree left, Tree right)
{
  Tree t = malloc(sizeof(*t));

  if (!t) {
    fprintf(stderr, "[tree_mk]: memory allocation failed.\n");
    exit(1);
  }
  T_TYPE (t) = type;
  T_LEFT (t) = left;
  T_RIGHT(t) = right;
  return t;
}

/* Prints tree `t' in infix to stdout with parens around '+' and '&'. */
void
tree_print(Tree t)
{
  if (!t) return;

  switch (T_TYPE(t)) {
  case '!':
    printf("!");
    tree_print(T_LEFT(t));
    break;

  case '&':
    /*FALL THROUGH*/
  case '+':
    printf("(");
    tree_print(T_LEFT(t));
    printf(" %c ", T_TYPE(t));
    tree_print(T_RIGHT(t));
    printf(")");
    break;

  case ';':
    tree_print(T_LEFT(t));
    tree_print(T_RIGHT(t));
    break;

  case '@':
    printf("@ ");
    tree_print(T_LEFT(t));
    printf(" = ");
    tree_print(T_RIGHT(t));
    printf(";\n");
    break;

  case '0':
    /*FALL THROUGH*/
  case '1':
    /*FALL THROUGH*/
  default:			/* Variable */
    printf("%c", T_TYPE(t));
    break;
  }
}
