/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Kedarnath Balakrishnan, bala@nec-labs.com */
#ifndef DEFINE_H
#define DEFINE_H

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

#define MAXLINE 500              /* Input buffer size */
#define MAXNAME 31               /* File name size */
#define FAILURE 0
#define SUCCESS 1                /* Return values for podem */
#define ABORTION 2 
#define CONTINUE -1

#define NORMAL   1
#define FAULTSIM 2
#define INFINITE 0xffffffff
#define FLT_INF 1.0E20

#define Isbool(x) (((x) == '0' || (x) == '1')? 1 : 0)
#define Isdb(x) (((x) == 'D' || (x) == 'B')? 1 : 0)

#define INTSIZE (sizeof(int) * 8)                 /* Integer size in bits */
#define Word(nl) ((nl)/INTSIZE)                   /* Word and Bit give the */
/*
#define Bit(nl) (INTSIZE - ((nl) % INTSIZE) - 1)   bit position for nl */
#define Bit(nl) ((nl) % INTSIZE)                  /* bit position for nl */

/*---------------- Set operation macro definition ------------------------*/
#define Bitset(fp, ln) ((fp)[Word(ln)] & (1 << Bit(ln)))
#define Setbit(fp, ln) (fp)[Word(ln)] |= (1 << Bit(ln))
#define Clearbit(fp, ln) (fp)[Word(ln)] &= ~(1 << Bit(ln))
#define Funion(i, fp1, fp2, fp3) \
   for(i = 0; i<Nints; i++) (fp1)[i] = (fp2)[i] | (fp3)[i]
#define Fintersect(i, fp1, fp2, fp3) \
   for(i = 0; i<Nints; i++) (fp1)[i] = (fp2)[i] & (fp3)[i]
#define Fcompliment(i, fp1, fp2) \
   for(i = 0; i<Nints; i++) (fp1)[i] = ~(fp2)[i]
#define Fcopy(i, fp1, fp2) \
   for(i = 0; i<Nints; i++) (fp1)[i] = (fp2)[i]
#define Fxor(i, fp1, fp2, fp3) \
   for(i = 0; i<Nints; i++) (fp1)[i] = (fp2)[i] ^ (fp3)[i]
#define Zeros(i, fp) \
   for(i = 0; i<Nints; i++) (fp)[i] = 0
#define Ones(i, fp) \
   for(i = 0; i<Nints; i++) (fp)[i] = ~0
/*------------------------------------------------------------------------*/

enum e_ntype {GATE, PI, FB, PO};    /* column 1 of circuit format */
enum e_gtype {IPT, BRCH, XOR, OR, NOR, NOT, NAND, AND};   /* gate types */

struct c_struct {             /* controllability cost structure */
   unsigned one;              /* one controllability cost */
   unsigned zero;             /* zero controllability cost */
};

typedef struct n_struc {
   unsigned indx;             /* node index(from 0 to NumOfLine - 1 */
   unsigned num;              /* line number(May be different from indx */
   enum e_gtype type;         /* gate type */
   unsigned fin;              /* number of fanins */
   unsigned fout;             /* number of fanouts */
   struct n_struc **unodes;   /* pointer to array of up nodes */
   struct n_struc **dnodes;   /* pointer to array of down nodes */
   int level;                 /* level of the gate output */
   char val;                  /* Boolean value after fsim */
   struct c_struct c_cost;    /* controllability cost */
   struct c_struct o_cost;    /* observability cost */
   int *flist;                /* fault list, one bit for each line */
} NSTRUC;

typedef struct {              /* target fault and objective structure */
   NSTRUC *node;              /* for podem */
   char val;
} VSTRUC;

typedef struct node_list_str { /* linked list structure */
   NSTRUC *ptr;                          
   struct node_list_str *next; /* next element */
} node_list;


typedef struct fault_list_str { /* linked list for target fault */
   NSTRUC *node;                /* faulty node */
   char sa01;                   /* stuck-at-v */
   struct fault_list_str *next;
   struct fault_list_str *prev;
} fault_list;

/* Some of the routines available to you. Detailed documentation
   is available in the respective source files
*/

/* ckt.c */
int cread(char *cp);
int read_fault(FILE * ff);
int find_seq(unsigned maxlevel);
int print_stat(void);
unsigned int levelize(void);

/* podem.c */
int ctrl_cost(void);
int ctrl0_cost(NSTRUC *);
int ctrl1_cost(NSTRUC *);
int obs_cost(void);
int print_cost(void);
fault_list *best_target(void);
int imply(VSTRUC);
char gimply(NSTRUC *);
int testgen(fault_list *fault, char vec[]);
int podem(void);

int pivector(char vec[]);
int test(void);
int observ_cost(NSTRUC *);
int obsv_cost_and(NSTRUC *);
int obsv_cost_or(NSTRUC *);
int po_unknown(void);
char control_val(NSTRUC *);
int fill_dont_care(char vector[]);

/* sim.c */
unsigned int fsim(void);
int tsim(char *vec);
int remove_fault(fault_list *);
int drop_fault(void);
int print_detected_faults(void);
int print_undetected_faults(void);

#ifdef __cplusplus
}
#endif

#endif /* DEFINE_H */
