/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Kedarnath Balakrishnan, bala@nec-labs.com
*/

/* This is main.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "define.h"

VSTRUC Tgfault;			/* target fault */
NSTRUC *Node;			/* dynamic array of nodes */
NSTRUC **Pinput;		/* pointer to array of primary inputs */
NSTRUC **Poutput;		/* pointer to array of primary outputs */
NSTRUC **seq;			/* seqnal propagation order */
node_list *Dfront;		/* D-frontier */
unsigned Nnodes;		/* number of nodes */
unsigned Npi;			/* number of primary inputs */
unsigned Npo;			/* number of primary outputs */
unsigned Ngate;			/* number of gates */
int Nints;			/* number of integers for fault list */
int Nflt;			/* number of faults in the fault file */
int *Fl;			/* temprary fault list used in fsim */
int *det_sa1_fault;		/* s-a-1 fault set detected */
int *det_sa0_fault;		/* s-a-0 fault set detected */
fault_list *fault_set;		/* target fault list */

/* global varibales */
unsigned max_btrack;		/* maximum number of backtrack */
unsigned btrack;		/* number of backtracking */
unsigned tot_btrack;		/* total number of backtracking */
unsigned tot_det;		/* total number of detected faults */
unsigned udet;			/* number of undetectable faults */
unsigned aborted;		/* number of aborted faults */

/*-----------------------------------------------------------------------
input: nothing
output: nothing
called by: shell
description:
This is the main program of the simulator. It displays the prompt, reads
and parses the user command, and calls the corresponding routines.
Commands not reconized by the parser are passed along to the shell.
The command is executed according to some pre-determined sequence.
For example, we have to read in the circuit description file before any
action commands.  
-----------------------------------------------------------------------*/
int
main(int argc, char *argv[])
{

  FILE *ff, *vecfile;
  int i, j;
  unsigned int k;
  unsigned maxlevel;
  int numvec = 1;
  unsigned maxnode;
  char *vector;
  char ckt[MAXLINE];
  fault_list new, *temp;
  int mode = 0;

  max_btrack = INFINITE;

  if (argc == 4) {
    strcpy(ckt, argv[1]);
  } else {
    fprintf(stderr, "Usage: testgen cktfile faultfile vectorfile\n");
    exit(-1);
  }

  /* read circuit */
  maxnode = cread(ckt);
  Ngate = Nnodes - Npi;

  /* open fault file */
  if ((ff = fopen(argv[2], "r")) == NULL) {
    fprintf(stderr, "File %s does not exist!\n", argv[2]);
    exit(-1);
  }

  /* open vector file to write */
  if ((vecfile = fopen(argv[3], "w")) == NULL) {
    fprintf(stderr, "Can't open file %s to write!\n", argv[3]);
  }

  maxlevel = levelize();

  seq = (NSTRUC **) malloc(Ngate * sizeof(NSTRUC *));

  det_sa0_fault = (int *) malloc(Nints * sizeof(int));
  Ones(i, det_sa0_fault);
  det_sa1_fault = (int *) malloc(Nints * sizeof(int));
  Ones(i, det_sa1_fault);

  new.node = NULL;
  find_seq(maxlevel);

  /* allocate vector value */
  vector = (char *) malloc(Npi + 1);

  /* Initally all primary inputs are assumed to be '0' */
  for (k = 0; k < Npi; k++)
    vector[k] = 'X';
  vector[Npi] = '\0';
  tsim(vector);			/* Each node has its initial value */
  tot_btrack = tot_det = 0;

  /* read fault descript file */
  read_fault(ff);

  /* initialize random ptr */
  srand48(42);
  /* output arguments */

  printf("\ntestgen");
  for (i = 1; i < argc; i++)
    printf(" %s", argv[i]);
  printf("\n");

  /* calculate controllability/observability costs */
  ctrl_cost();
  obs_cost();

  /* print_cost() ; */

  while (1) {			/* ATPG routine */
    temp = fault_set;
    /* select easiest fault */
    temp = best_target();

    /* printing output */

    new.node = temp->node;
    new.sa01 = temp->sa01;
    remove_fault(temp);

    btrack = 0;

    /* run atpg */
    i = testgen(&new, vector);
    tot_btrack += btrack;

    if ((i & 3) == SUCCESS) {	/* detectable fault */
      numvec++;
      j = 1;
      /* random fill remaining bits */
      fill_dont_care(vector);
      fprintf(vecfile, "%s \n", vector);
      tsim(vector);

      j += fsim();
      tot_det += j;

      /* dropping detected faults */
      drop_fault();

    } else if ((i & 3) == FAILURE) {
      /* Undetectable fault */
      udet++;
    } else if ((i & 3) == ABORTION) {
      /* test generation aborted fault */
      aborted++;
    }

    if (fault_set == NULL) {
      break;
    }
  }

  printf("\n");
  printf("Total vectors: %u\n", numvec - 1);
  printf("Total number of faults: %u\n", Nflt);
  printf("Undetectable faults: %u\n", udet);
  if (!mode)
    printf("Aborted faults: %u\n", aborted);
  printf("Fault coverage: %3.2f%s (%u)\n", 100.0 * tot_det / (Nflt - udet),
	 "%", tot_det);
  return 0;
}

/*========================= End of program ============================*/
