/* Copyright (c) 2005 ACM/SIGDA
   Prepared by Kedarnath Balakrishnan, bala@nec-labs.com
*/

/* This is podem.c */
#include <stdlib.h>
#include "define.h"

extern unsigned btrack;
extern unsigned max_btrack;
extern int Nnodes;
extern int Ngate;
extern NSTRUC **seq;
extern NSTRUC *Node;
extern VSTRUC Tgfault;		/* target fault */
extern node_list *Dfront;	/* D-frontier */
extern int Nnodes;		/* number of nodes */
extern int Npi;			/* number of primary inputs */
extern int Npo;			/* number of primary outputs */
extern NSTRUC **Pinput;		/* pointer to array of primary inputs */
extern NSTRUC **Poutput;	/* pointer to array of primary outputs */
extern fault_list *fault_set;

VSTRUC objective(void);
VSTRUC backtrace(VSTRUC);
struct c_struct obsv_cost_branch(NSTRUC *);

/*-----------------------------------------------------------------------
input: faulty line and value
output: nothing
called by: main
description:
The routine is an interface between "main" routine and actual recursive
"podem" routine. It checks for legal arguments, sets the target fault,
prints herald, and calls recursive "podem" routine.
-----------------------------------------------------------------------*/
int
testgen(fault_list *fault, char vec[])
{
  int i, res;
  char val;

  val = fault->sa01;
  if (val == '0' || val == '1') {
    Tgfault.node = fault->node;
    Tgfault.val = fault->sa01;
    Dfront = NULL;

    for (i = 0; i < Nnodes; i++)
      Node[i].val = 'X';

    res = podem();
    if (res == FAILURE)
      return (FAILURE);
    else if (res == ABORTION)
      return (ABORTION);
    else {
      pivector(vec);
      return (SUCCESS);
    }
  }
  return 0;
}

/*-----------------------------------------------------------------------
input: nothing
output: return code for test generation process
called by: testgen 
description:
-----------------------------------------------------------------------*/
int
podem()
{
  int rtest, res;
  VSTRUC obj, piassign;

  if ((rtest = test()) != CONTINUE)
    return (rtest);
  obj = objective();
  piassign = backtrace(obj);
  imply(piassign);
  if ((res = podem()) == SUCCESS)
    return (SUCCESS);
  else if (res == ABORTION)
    return (ABORTION);
  piassign.val ^= 1;
  imply(piassign);
  if ((res = podem()) == SUCCESS)
    return (SUCCESS);
  else if (res == ABORTION)
    return (ABORTION);
  piassign.val = 'X';
  btrack++;
  imply(piassign);
  if (btrack > max_btrack)
    return (ABORTION);
  else
    return (FAILURE);
}

/*-----------------------------------------------------------------------
input: nothing
output: an objective line and value
called by: main, podem
description:
The routine returns an objective line and value to the caller.
If the target fault is l s-a-v and the value on line l is 'X', then
(l, /v) is the objective. Otherwise, a gate G from the D-frontier and
an input j of G with the value 'X' is selected. If c is controlling
value of G, then (j, c) is the objective. If the D-frontier is empty
after the fault has been activated, no objective is returned.
-----------------------------------------------------------------------*/
VSTRUC
objective()
{
  unsigned int i;
  char ncv;
  VSTRUC obj;
  NSTRUC *np;

  np = Tgfault.node;
  if (np->val == 'X')
    ncv = Tgfault.val ^ 1;
  else {
    if (Dfront != NULL) {
      np = Dfront->ptr;
      i = 0;
      while (i < np->fin && (np->unodes[i]->val != 'X'))
	i++;
      if (np->type == AND || np->type == NAND)
	ncv = '1';
      else if (np->type == OR || np->type == NOR)
	ncv = '0';
      np = np->unodes[i];
    } else {
      printf("No objective under current PI assignment.\n");
      return (obj);
    }
  }
  obj.node = np;
  obj.val = ncv;
  return (obj);
}

/*-----------------------------------------------------------------------
input: a string of line name and the objective value on the line
output: a possible PI assignment to satisfy the objective
called by: main, podem
description:
The routine backtraces to the PI's to find a possible PI assignment
to satisfy the objective line value. It first checks if the line exists,
then it traces any input line not yet assigned a value and counts the
inversion parity until a PI is reached. The routine simply choose the
first path with all 'X' without any cost consideration.
The routine also adds the PI assignment found to the decision tree
used in "itest".
Ref: Fig. 6.27 in the text book.
-----------------------------------------------------------------------*/
VSTRUC
backtrace(VSTRUC obj)
{
  unsigned int i;
  NSTRUC *np;
  char val;
  VSTRUC pp;

  np = obj.node;
  val = obj.val;
  while (np->type != IPT) {
    i = 0;
    while (i < np->fin && np->unodes[i]->val != 'X')
      i++;
    if (np->type == NAND || np->type == NOR || np->type == NOT)
      val ^= 1;
    np = np->unodes[i];
  }
  pp.node = np;
  pp.val = val;
  return (pp);
}

/*-----------------------------------------------------------------------
input: nothing
output: SUCCESS if a test is found
FAILURE if a test is not possible with current PI assignment
CONTINUE otherwise
called by: podem
description:
The routine prints out the test vector if a 'D' or 'B' appears on
any primary output and returns SUCCESS to the caller.
It returns FAILURE if the target fault can not be activated or the
D-frontier is empty before any fault effect propagates to the primary
outputs under current PI assignment. This does not mean the fault is
undetectable, but that backtrack is required to find the test or to draw
to the conclusion that the fault is undetectable.
Otherwise, it returns CONTINUE to the caller demanding further action.
-----------------------------------------------------------------------*/
int
test()
{
  int i, j;
  char val;

  val = Tgfault.node->val;
  for (i = 0, j = 0; i < Npo; i++)
    if (!Isdb(Poutput[i]->val))
      j++;

  if (j < Npo)
    return (SUCCESS);
  else if (Isbool(val) || (Isdb(val) && Dfront == NULL))
    return (FAILURE);
  else
    return (CONTINUE);
}

/*-----------------------------------------------------------------------
input: nothing
output: nothing
called by: testgen
description:
This routine prints out current line values at the PI's. 'D' and 'B' are
interpreted as '1' and '0' respectively.
-----------------------------------------------------------------------*/
int
pivector(char vec[])
{
  int i, j = 0;
  char val;

  for (i = 0; i < Npi; i++) {
    val = Pinput[i]->val;
    if (val == 'D')
      val = '1';
    else if (val == 'B')
      val = '0';
    else if (val == 'X')
      j++;

    vec[i] = val;
  }
  vec[Npi] = '\0';
  return (j);
}

/*-----------------------------------------------------------------------
input: nothing
output: nothing
called by: main
description:
find whether a PO has unknown value or not
-----------------------------------------------------------------------*/
int
po_unknown()
{
  int i;

  for (i = 0; i < Npo; i++)
    if (Poutput[i]->val == 'X')
      return (1);

  return 0;
}

/*-----------------------------------------------------------------------
input : nothig
output: nothing 
called by: main
description:
This routine calculates inital controllability cost 
------------------------------------------------------------------------*/
int
ctrl_cost()
{
  int i;

  for (i = 0; i < Npi; i++) {
    ctrl1_cost(Pinput[i]);
    ctrl0_cost(Pinput[i]);
  }

  for (i = 0; i < Ngate; i++) {
    ctrl1_cost(seq[i]);
    ctrl0_cost(seq[i]);
  }
  return 0;
}

/*-----------------------------------------------------------------------
input : a pointer to a node
output: nothing 
called by: main
description:
This routine calculate one controllability cost value given a node.
------------------------------------------------------------------------*/
int
ctrl1_cost(NSTRUC *np)
{
  unsigned i, j, sum, fout, cost;

  /* compute one controllability cost */
  sum = 0;
  if (np->fout == 0)
    fout = 1;
  else
    fout = np->fout;

  switch (np->type) {
  case IPT:
    cost = fout - 1;
    break;
  case NOT:
    cost = np->unodes[0]->c_cost.zero + fout - 1;
    break;
  case BRCH:
    cost = np->unodes[0]->c_cost.one;
    break;
  case AND:
    for (i = 0; i < np->fin; i++)
      sum += np->unodes[i]->c_cost.one;
    cost = sum + fout - 1;
    break;
  case NAND:
    sum = np->unodes[0]->c_cost.zero;
    for (i = 1; i < np->fin; i++)
      if (sum > (j = np->unodes[i]->c_cost.zero))
	sum = j;
    cost = sum + fout - 1;
    break;
  case OR:
    sum = np->unodes[0]->c_cost.one;
    for (i = 1; i < np->fin; i++)
      if (sum > (j = np->unodes[i]->c_cost.one))
	sum = j;
    cost = sum + fout - 1;
    break;
  case NOR:
    for (i = 0; i < np->fin; i++)
      sum += np->unodes[i]->c_cost.zero;
    cost = sum + fout - 1;
    break;
  default:
    cost = 0;
  }
  np->c_cost.one = cost;
  return 0;
}

/*-----------------------------------------------------------------------
input : a pointer to a node
output: nothing 
called by: main
description:
This routine calculate zero controllability cost value given a node.
------------------------------------------------------------------------*/
int
ctrl0_cost(NSTRUC *np)
{
  unsigned i, j, sum, fout, cost;

  /* compute zero controllability cost */
  sum = 0;
  switch (np->type) {
  case IPT:
    cost = fout - 1;
    break;
  case NOT:
    cost = np->unodes[0]->c_cost.one + fout - 1;
    break;
  case BRCH:
    cost = np->unodes[0]->c_cost.zero;
    break;
  case AND:
    sum = np->unodes[0]->c_cost.zero;
    for (i = 1; i < np->fin; i++)
      if (sum > (j = np->unodes[i]->c_cost.zero))
	sum = j;
    cost = sum + fout - 1;
    break;
  case NAND:
    for (i = 0; i < np->fin; i++)
      sum += np->unodes[i]->c_cost.one;
    cost = sum + fout - 1;
    break;
  case OR:
    for (i = 0; i < np->fin; i++)
      sum += np->unodes[i]->c_cost.zero;
    cost = sum + fout - 1;
    break;
  case NOR:
    sum = np->unodes[0]->c_cost.one;
    for (i = 1; i < np->fin; i++)
      if (sum > (j = np->unodes[i]->c_cost.one))
	sum = j;
    cost = sum + fout - 1;
    break;
  default:
    cost = 0;
  }
  np->c_cost.zero = cost;
  return 0;
}

/*-----------------------------------------------------------------------
input : nothing
output: nothing
called by: main
description:
This routine that calculates observability cost for all nodes
------------------------------------------------------------------------*/
int
obs_cost()
{
  int i;
  NSTRUC *np;

  for (i = (Ngate - 1); i >= 0; i--) {
    np = seq[i];
    /* observability cost of all PO is zero */
    if (np->fout)
      observ_cost(np);
  }
  /* calculate observability cost for all PIs */
  for (i = 0; i < Npi; i++) {
    np = Pinput[i];
    observ_cost(np);
  }
  return 0;
}

/*-----------------------------------------------------------------------
input : a pointer to a node, the implied value on the node
output:  nothing
called by: obs_cost
description:
This is routine that calculate observability cost value given a node,
and the implied value on the node.
------------------------------------------------------------------------*/
int
observ_cost(NSTRUC *np)
{
  NSTRUC *pp;
  unsigned cost;
  enum e_gtype gate;
  struct c_struct min;

  pp = np->dnodes[0];
  gate = pp->type;

  if (gate == AND || gate == NAND)
    cost = obsv_cost_and(np);
  else if (gate == OR || gate == NOR)
    cost = obsv_cost_or(np);
  else
    cost = 0;

  if (gate == BRCH) {		/* node np is a fanout stem */
    min = obsv_cost_branch(np);
    np->o_cost.one = min.one;
    np->o_cost.zero = min.zero;
  } else {
    np->o_cost.one = cost + pp->o_cost.one;
    np->o_cost.zero = cost + pp->o_cost.zero;
  }
  return 0;
}

/*------------------------------------------------------------------------
input : a pointer to anode in the array node
output: observability cost value for AND/NAND gate.
called by: observ_cost
description:
The routine calculates observability cost O(i) as:

O(i) = O(x) + SUM C01(j)
j!=i

Where, j is the inputs of which previous value is controlling value (0)
-------------------------------------------------------------------------*/
int
obsv_cost_and(NSTRUC *np)
{
  unsigned int i;
  int ctrlty_sum;
  NSTRUC *pp, *qq;

  pp = np->dnodes[0];
  ctrlty_sum = 0;
  for (i = 0; i < pp->fin; i++) {
    qq = pp->unodes[i];
    if (np != qq)
      ctrlty_sum += qq->c_cost.one;
  }
  return (ctrlty_sum);
}

/*------------------------------------------------------------------------
input : a pointer to a node in the array node
output: observability cost value for OR/NOR gate
called by: observ_cost
description:
The routine caculates observability cost O(i) as:

O(i) = O(x) + SUM C0(j)
j!=i

Where, j is the inputs of which previous value is controlling value (1)
-------------------------------------------------------------------------*/
int
obsv_cost_or(NSTRUC *np)
{
  unsigned int i;
  int ctrlty_sum;
  NSTRUC *pp, *qq;

  pp = np->dnodes[0];
  ctrlty_sum = 0;
  for (i = 0; i < pp->fin; i++) {
    qq = pp->unodes[i];
    if (np != qq)
      ctrlty_sum += qq->c_cost.zero;
  }
  return (ctrlty_sum);
}

/*------------------------------------------------------------------------
input : a pointer to anode
output: observability cost for stem with branches
called by: observ_cost
description:
The routine does calculate the observability cost O(x) as:

O(x) = min { O(j) } + SUM O(i)
j=1,n          i!=j

Where, i are the gates of which all inputs except x have non controlling
value.
-------------------------------------------------------------------------*/
struct c_struct
obsv_cost_branch(NSTRUC *np)
{
  unsigned int i;
  unsigned int min;
  NSTRUC *pp;
  struct c_struct cost;

  min = (np->dnodes[0])->o_cost.zero;
  for (i = 1; i < np->fout; i++) {
    pp = np->dnodes[i];
    if (min > pp->o_cost.zero)
      min = pp->o_cost.zero;
  }
  cost.zero = min;
  cost.one = min;
  return (cost);
}

/*------------------------------------------------------------------------
input : a pointer to linked list for target faults
output: pointer to node which has minimum cost
called by: main
description:
The routine caculates test generation cost as
if target fault = s-a-1
cost = C0(i) + 0(i)
else if target fault = s-a-0
cost = C1(i) + 0(i)
Where, i is node number for the fault site
-------------------------------------------------------------------------*/

fault_list *
best_target()
{
  fault_list *temp, *min;
  char f_val;
  unsigned min_cost;
  unsigned i, cost;
  NSTRUC *np;

  temp = fault_set;
  min_cost = INFINITE;
  for (i = 1; temp != NULL; i++) {
    np = temp->node;
    f_val = temp->sa01;
    if (f_val == '1')
      cost = np->c_cost.zero + np->o_cost.zero;
    else if (f_val == '0')
      cost = np->c_cost.one + np->o_cost.one;

    if (cost < min_cost) {
      min_cost = cost;
      min = temp;
    }
    temp = temp->next;
  }

  return (min);
}

/*------------------------------------------------------------------------
input : none
output: none
called by: main
description: prints controllability/observability cost of each node
-------------------------------------------------------------------------*/

int
print_cost()
{
  int i;

  for (i = 0; i < Nnodes; i++) {
    printf("%3d: ", Node[i].num);
    printf("TC1=%d  ", Node[i].c_cost.one);
    printf("TC0=%d  ", Node[i].c_cost.zero);
    printf("TO1=%d  ", Node[i].o_cost.one);
    printf("TO0=%d  ", Node[i].o_cost.zero);
    printf("TG1=%d  ", Node[i].o_cost.one + Node[i].c_cost.one);
    printf("TG0=%d  ", Node[i].o_cost.zero + Node[i].c_cost.zero);
    printf("\n");
  }
  return 0;
}

/*------------------------------------------------------------------------
input : node pointer
output: char - controlling value
called by: none
description: returns controlling value of gate
-------------------------------------------------------------------------*/

char
control_val(NSTRUC *np)
{
  switch (np->type) {
  case NOT:
    return ('X');
  case AND:
    return ('0');
  case NAND:
    return ('0');
  case OR:
    return ('1');
  case NOR:
    return ('1');
  default:
    printf("Something nasty happened in control_val\n");
    exit(-1);
  }
}

/*------------------------------------------------------------------------
input : nothing
output: nothing
called by: main
description: fill don't care input with binary value randomly
-------------------------------------------------------------------------*/
int
fill_dont_care(char vector[])
{
  int i, j;
  char val;
  double rndval;

  for (i = 0; i < Npi; i++) {
    val = vector[i];
    if (val == 'X') {
      rndval = drand48();
      j = rndval + 0.5;
      vector[i] = j + '0';
    }
  }
  return 0;
}
